#!/usr/bin/python3

import os
from roborecipe.data_type import *
from roborecipe.ImageGenerator import *
from roborecipe.StlLoader import *

class ViewListGenerator:
	def __init__(self, dl, rt):
		self.dl = dl
		self.rt = rt
	def GetViewList(self, output_dir):
		render_view_list=[]

		for render_target_component in self.dl:
			if type(render_target_component) is not DataAssembly:
				continue

			render_list = self.rt.GetItemListWithTransform(render_target_component.id)

			component_output_dir = output_dir+"/"+render_target_component.id.pkg_name+'/'+render_target_component.id.type_name
			os.makedirs(component_output_dir, exist_ok=True)

			for step_seq_no in range(len(render_target_component.step_list)):
				step_render_list = []
				for i in render_list:
					# print(i.component.id.getName())
					# print(i.component.stl_path)
					# print(i.GetWholeTransform())
					# print(i.step)
					if i.step <= step_seq_no:
						p1 = PartSource()
						p1.triangles = StlLoader(i.component.stl_path).get_triangles()
						wtr=i.GetWholeTransform()
						p1.position = [wtr.position.x,wtr.position.y,wtr.position.z]
						p1.rotation = wtr.rotation.getRPY()
						if i.step == step_seq_no:
							p1.move = i.move
						step_render_list.append(p1)

				for view_seq_no in range(len(render_target_component.step_list[step_seq_no].view_list)):
					view_data = render_target_component.step_list[step_seq_no].view_list[view_seq_no]
				
					render_view = ViewSource()
					render_view.output_filepath = component_output_dir + '/asm_'+str(step_seq_no)+'_'+str(view_seq_no)+'.gif'
					render_view.step=step_seq_no
					render_view.part_list = step_render_list
					render_view.look_from = view_data.look_from
					render_view.look_at = view_data.look_at
					render_view.step=10
					render_view_list.append(render_view)
		return render_view_list

if __name__ == '__main__':
	from XmlParser import *
	from TreeAnalyzer import *
	from DirectorySearch import *
	from ImageGenerator import *
	from StlLoader import *

	ds = DirectorySearch('../sample')
	path_pair_list = ds.getComponentPathPairList()

	component_list = ComponentListParser(path_pair_list).getList()

	ta = TreeAnalyzer(component_list, ComponentIdentifier('sample_project', 'main_asm'))

	print("#### quantity ####")
	ql = ta.getQuantityList()
	for c in ql:
		print(c.id.getName(), ql[c])

	print("#### depend order ####")
	dl = ta.getDependOrderList()

	print("#### RenderList ####")
	rt = RenderTree(component_list, dl)

	vlg = ViewListGenerator(dl, rt)
	output_dir ="/home/ubuntu/roborecipe/out"
	render_view_list = vlg.GetViewList(output_dir)

	ig = ImageGenerator(sys.argv)

	ig.renderViewList(render_view_list)



#!/usr/bin/python3
from roborecipe.DirectoryLoader import * 
from roborecipe.AssemblyAnalyzer import * 
from roborecipe.TopPageGenerator import * 
from roborecipe.PartsPageGenerator import *
from roborecipe.PartImageGenerator import *

if __name__ == '__main__':
    dl = DirectoryLoader('../sample')
    out_ds = OutputDirectorySearch('../out')
    component_path_list = []
    for component in dl.get_component_path_list(): 
        component_path_list.append(component)

    cll = ComponentListLoader()
    for path_item in component_path_list:
        cll.load(path_item)

    top_assembly_id = ComponentIdentifier('sample_project', 'main_asm')
    ata = AssemblyTreeAnalyzer(cll, top_assembly_id)
    ada = AssemblyDependencyAnalyzer(cll, top_assembly_id)

    # top page
    tg = TopPageGenerator(top_assembly_id, ata, ada)
    tg.write('../out')

    # package copy
    used_pkg_name_set = ada.get_pkg_name_set()
    dl.copy_resouces_dir(used_pkg_name_set, '../out')

    # part page (screw)
    for item in ata.get_quantity_list(ComponentCategoryEnum.SCREW):
        # image
        mpig = MechanicalPartsImageGenerator(item.comp)
        if out_ds.need_update_image(mpig.get_pkg_name(), mpig.get_m_datetime()):
            mpig.write('../out')
        else:
            print('skip image')

        # html
        mpg = MechanicalPartsPageGenerator(item.comp)
        mpg.write('../out')

    # part page (laser_cut)
    for item in ata.get_quantity_list(ComponentCategoryEnum.LASER_CUT):
        # image
        mpig = MechanicalPartsImageGenerator(item.comp)
        if out_ds.need_update_image(mpig.get_pkg_name(), mpig.get_m_datetime()):
            mpig.write('../out')
        else:
            print('skip image')

        # html
        mpg = ProcessPartsPageGenerator(item.comp)
        mpg.write('../out')

    # part page (assembly)
    for item in ada.get_list():
        # image
        ata = AssemblyTreeAnalyzer(cll, item.comp.id)
        aig = AssemblyImageGenerator(item.comp, ata.get_position_list(), ata.get_step_view_list())
        asm_m_datetime = AssemblyDependencyAnalyzer(cll, item.comp.id).get_m_datetime()

        if out_ds.need_update_image(aig.get_pkg_name(), asm_m_datetime):
            aig.write('../out')
        else:
            print('skip image')

        # html
        apg = AssemblyPageGenerator(item.comp)
        apg.write('../out')


#!/usr/bin/python3

import collections
import networkx as nx
import copy

from roborecipe.data_type import *
from roborecipe.ComponentLoader import *

class QuantityItem:
    def __init__(self, comp, quantity):
        self.comp = comp
        self.quantity = quantity

class AssemblyDependencyAnalyzer:
    def __init__(self, component_list, top_assembly_id):
        self.component_list = component_list
        # dependency
        el = self._get_edge_list(top_assembly_id)
        self.dependency_list = self._get_dependency_order(el)
        # quantity
        self.assembly_list = self._get_assembly_list(top_assembly_id)

    def get_list(self):
        quantity_dir = collections.Counter(self.assembly_list)
        asm_list = []
        for comp in self.dependency_list:
            if comp.category.is_assembly():
                asm_list.append(QuantityItem(comp, quantity_dir[comp.id]))
        return asm_list

    def get_pkg_name_set(self):
        pkg_name_list = []
        for comp_id in self.assembly_list:
            pkg_name_list.append(comp_id.pkg_name)
        return set(pkg_name_list)

    def get_m_datetime(self):
        m_datetime_list = []
        for comp_id in self.assembly_list:
            comp = self.component_list.getComponent(comp_id)
            m_datetime_list.append(comp.m_datetime)
        return max(m_datetime_list)

    def _get_edge_list(self, component_id, parent=None):
        edge_list = []
        comp = self.component_list.getComponent(component_id)
        if comp is None:
            RED = '\033[31m'
            END = '\033[0m'
            print(RED + component_id.getName() + ' is empty' + END)
            return edge_list
        if parent is not None:
            edge_list.append([parent, comp])
        if comp.category.is_assembly():
            for step in comp.procedure_list:
                for c in step.component_list:
                    el = self._get_edge_list(c.id, comp)
                    edge_list.extend(el)
        return edge_list

    def _get_dependency_order(self, edge_list):
        graph = nx.DiGraph()
        for e in edge_list:
            graph.add_edge(e[1], e[0])
        T = nx.topological_sort(graph)
        return list(T)

    def _get_assembly_list(self, component_id):
        asm_list = []
        comp = self.component_list.getComponent(component_id)
        if comp is None:
            RED = '\033[31m'
            END = '\033[0m'
            print(RED + component_id.getName() + ' is empty' + END)
            return edge_list

        asm_list.append(comp.id)
        if comp.category.is_assembly():
            for step in comp.procedure_list:
                for c in step.component_list:
                    l = self._get_assembly_list(c.id)
                    asm_list.extend(l)
        return asm_list

class AssemblyTreeItem:
    def __init__(self, id, step_no, comp_no):
        self.id = id
        self.step_no = step_no
        self.comp_no = comp_no
    def __repr__(self):
        return self.id.getName()+'['+str(self.step_no)+','+str(self.comp_no)+']'

class AssemblyDisplayObject:
    def __init__(self, comp, step_no, transform, move):
        self.comp = comp
        self.step_no = step_no
        self.transform = transform
        self.move = move
    def __repr__(self):
        return '[' + str(self.step_no) + '] ' + self.comp.id.getName() + ' ' + str(self.transform) + ' ' + str(self.move)

class AssemblyTreeAnalyzer:
    def __init__(self, component_list, top_assembly_id):
        self.component_list = component_list
        self.tree_list = self._get_transform_tree(top_assembly_id)
        self.step_view_list = self._extract_step_view_list(top_assembly_id)

    def get_position_list(self):
        output_list = []
        for item_list in self.tree_list:
            transform = Transform()
            for item in item_list:
                assembly_item = self.component_list.getComponent(item.id)
                if (assembly_item is not None) and (item.step_no is not None) and (item.comp_no is not None):
                    asm_transform = assembly_item.procedure_list[item.step_no].component_list[item.comp_no].transform
                    transform = asm_transform * transform; 

            top_asm_item = item_list[-1]
            top_assemnly_item = self.component_list.getComponent(top_asm_item.id)
            move = top_assemnly_item.procedure_list[top_asm_item.step_no].component_list[top_asm_item.comp_no].move
            tip_item = item_list[0]
            output_list.append(AssemblyDisplayObject(self.component_list.getComponent(item_list[0].id), top_asm_item.step_no, transform, move))
        return output_list

    def get_quantity_list(self, category_enum):
        part_list = []
        for item_id_list in self.tree_list:
            comp = self.component_list.getComponent(item_id_list[0].id)
            if comp.category.value == category_enum:
                part_list.append(comp)
        quantity_dir = collections.Counter(part_list)

        part_list = []
        for comp in quantity_dir:
            part_list.append(QuantityItem(comp, quantity_dir[comp]))
        def key_func(obj):
            return obj.comp.id.getName()
        return sorted(part_list, key=key_func)

    def get_step_view_list(self):
        return self.step_view_list

    def _get_transform_tree(self, component_id):
        output_list = []
        parent_comp = self.component_list.getComponent(component_id)
        if parent_comp is None:
            RED = '\033[31m'
            END = '\033[0m'
            print(RED + component_id.getName() + ' is empty' + END)
            return output_list
        if parent_comp.category.is_assembly():
            output_list = []
            for step_no, step in enumerate(parent_comp.procedure_list):
                for comp_no, comp in enumerate(step.component_list):
                    sub_list = self._get_transform_tree(comp.id)
                    current_item = AssemblyTreeItem(parent_comp.id, step_no, comp_no)
                    for item_list in sub_list:
                        item_list.append(current_item)
                    output_list.extend(sub_list)
            return output_list
        else:
            return [[AssemblyTreeItem(parent_comp.id, None, None)]]

    def _extract_step_view_list(self, component_id):
        output_list = []
        parent_comp = self.component_list.getComponent(component_id)
        if parent_comp is None:
            RED = '\033[31m'
            END = '\033[0m'
            print(RED + component_id.getName() + ' is empty' + END)
            return output_list
        for step in parent_comp.procedure_list:
            output_list.append(step.view_list)
        return output_list

if __name__ == '__main__':
    component_path_list = [
        ComponentPathItem('screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3/roborecipe/pan_10.yaml'),
        ComponentPathItem('screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3/roborecipe/hollow_spacer_20.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/bar_plate.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/base_plate.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/side_asm.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/main_asm.yaml'),
    ]

    cll = ComponentListLoader()
    for path_item in component_path_list:
        cll.load(path_item)

    ada = AssemblyDependencyAnalyzer(cll, ComponentIdentifier('sample_project', 'main_asm'))

    print('#### Assembly ####')
    for item in ada.get_list():
        print(item.comp.id.getName(), item.quantity)

    ata = AssemblyTreeAnalyzer(cll, ComponentIdentifier('sample_project', 'main_asm'))

    print('#### Screw ####')
    for item in ata.get_quantity_list(ComponentCategoryEnum.SCREW):
        print(item.comp.id.getName(), item.quantity)
    print('#### LaserCut ####')
    for item in ata.get_quantity_list(ComponentCategoryEnum.LASER_CUT):
        print(item.comp.id.getName(), item.quantity)
    print('#### END ####')


    for item in ata.get_position_list():
        print(item.step_no,item.comp.rigit_body.mesh_filepath)

    for view_list in ata.get_step_view_list():
        print(view_list)


#!/usr/bin/python3
import pathlib
import glob
import os
import datetime
import xml.etree.ElementTree as ET
import shutil

class PackagePathItem:
    def __init__(self, pkg_name, pkg_base_path):
        self.pkg_name = pkg_name
        self.pkg_base_path = pkg_base_path
    def __str__(self):
        return '(' + self.pkg_name + ', ' + self.pkg_base_path + ')'

class ComponentPathItem:
    def __init__(self, pkg_name, pkg_base_path, comp_file_path, m_datetime):
        self.pkg_name = pkg_name
        self.pkg_base_path = pkg_base_path
        self.comp_file_path = comp_file_path
        self.m_datetime = m_datetime
    def __str__(self):
        return '(' + self.pkg_name + ', ' + self.pkg_base_path + ', ' + self.comp_file_path + ', ' + str(self.m_datetime) + ')'

class DirectoryLoader:
    def __init__(self, root_path):
        target_directory = pathlib.Path(root_path).resolve()
        self.pkg_list = self._get_valid_package_list(target_directory)

        self.package_component_list = []
        for p in self.pkg_list:
            for c in self._get_component_list(p[1]):
                m_data_time = datetime.datetime.fromtimestamp(os.path.getmtime(c))
                self.package_component_list.append(ComponentPathItem(p[0], p[1], c, m_data_time))

    def get_component_path_list(self):
        return self.package_component_list

    def get_pkg_list(self):
        pkg_path_list = []
        for p in self.pkg_list:
            pkg_path_list.append(PackagePathItem(p[0], p[1]))
        return pkg_path_list

    def copy_resouces_dir(self, pkg_set, output_dir):
        for p in self.pkg_list:
            if p[0] in pkg_set:
                source_dir = p[1] + '/resources'
                target_dir = str(output_dir) + '/' + p[0] + '/resources'
                if os.path.isdir(source_dir):
                    print('copy ' + source_dir)
                    shutil.copytree(source_dir, target_dir, dirs_exist_ok=True)
            print('skip copy resouces: ' + p[0])

    def _get_valid_package_list(self, target_directory):
        package_path_list = glob.glob(str(target_directory) + '/*/package.xml', recursive = True)

        output_list = []
        for package_path in package_path_list:
            name = self._get_package_name(package_path)
            if name is not None:
                package_base_dir = str(pathlib.Path(package_path).parent)
                output_list.append([name, package_base_dir])
        return output_list

    def _get_package_name(self, package_path):
        tree = ET.parse(package_path)
        elem = tree.getroot().find("name")
        if elem is not None:
            return elem.text
        RED = '\033[31m'
        END = '\033[0m'
        print(RED + package_path + ': <name> not found' + END)
        return None

    def _get_component_list(self, pkg_path):
        return glob.glob(pkg_path + '/roborecipe/*.yaml', recursive = True)

    def _get_modify_datetime(self, dir_path):
        file_list = glob.glob(dir_path + '/**/*', recursive = True)
        m_datetime_list = []
        for file_path in file_list:
            m_data_time = datetime.datetime.fromtimestamp(os.path.getmtime(file_path))
            m_datetime_list.append(m_data_time)
        return max(m_datetime_list)

class OutputDirectoryDatetimeItem:
    def __init__(self, pkg_name, image_m):
        self.pkg_name = pkg_name
        self.image_m = image_m

class OutputDirectorySearch:
    def __init__(self, output_base_dir):
        pkg_list = glob.glob(str(output_base_dir) + '/*/', recursive = True)
        self.image_datetime_dict = {}
        for pkg_path in pkg_list:
            m_datetime = self._get_m_datetime(pkg_path + 'images/')
            if m_datetime is not None:
                self.image_datetime_dict[pathlib.Path(pkg_path).name] = m_datetime

    def need_update_image(self, pkg_name, source_datetime):
        # print('######## ' + pkg_name + ' #########')
        if pkg_name not in self.image_datetime_dict:
            # print('not in list')
            return True
        else:
            # print('check ' + str(self.image_datetime_dict[pkg_name]) + ' < ' + str(source_datetime))
            return self.image_datetime_dict[pkg_name] < source_datetime

    def _get_m_datetime(self, dir_path):
        file_list = glob.glob(dir_path + '/**/*', recursive = True)
        m_datetime_list = []
        for file_path in file_list:
            m_data_time = datetime.datetime.fromtimestamp(os.path.getmtime(file_path))
            m_datetime_list.append(m_data_time)
        if file_list:
            return max(m_datetime_list)
        else:
            return None

if __name__ == '__main__':
    dl = DirectoryLoader('../sample')
    for component in dl.get_component_path_list(): 
        print(component)

    out_ds = OutputDirectorySearch('../out')
    print(out_ds.need_update_image('screw_m3', datetime.datetime(2023, 4, 1)))
    print(out_ds.need_update_image('screw_m3', datetime.datetime(2023, 10, 1)))
    print(out_ds.need_update_image('screw_m4', datetime.datetime(2023, 4, 1)))

    dl.copy_resouces_dir(set(['screw_m3','sample_project']), '../out')


from OpenGL.GL   import *
from OpenGL.GLU  import *
from OpenGL.GLUT import *
from PIL import Image
from PIL import ImageOps

class PartSource:
    def __init__(self):
        self.triangles = None
        self.position = [0,0,0]
        self.rotation = [0,0,0]
        self.move = [0,0,0,0]
        self.color = "gray"

class ViewSource:
    def __init__(self):
        self.output_filepath = ""
        self.part_list = []
        self.look_from = [200,0,0]
        self.look_at = [0,0,0]
        self.step = 0

class ImageGenerator:
    def __init__(self, argv):
        self.InitWindow(argv)
        self.target_view_list = []

    def renderViewList(self, view_list):
        self.target_view_list = view_list
        glutTimerFunc(100, self.timerProcess, 0)
        glutMainLoop()

    def InitWindow(self, argv):
        glutInit(argv)
        glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH)
        glutInitWindowSize(640, 640)
        glutInitWindowPosition(100, 100)
        glutCreateWindow("test")             # modify
        glClearColor(0.0, 0.0, 0.0, 0.0)
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_CULL_FACE)
        glCullFace(GL_BACK)

        # lighting
        whiteColor = [ 0.0, 0.8, 0.0, 1.0 ]
        grayColor = [ 0.2, 0.2, 0.2, 1.0 ]
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)
        glEnable(GL_COLOR_MATERIAL)
        glLightfv(GL_LIGHT0, GL_DIFFUSE, whiteColor)
        glLightfv(GL_LIGHT0, GL_SPECULAR, whiteColor)
        glLightfv(GL_LIGHT0, GL_AMBIENT, grayColor)

        glutDisplayFunc(self.display)
        glutReshapeFunc(self.reshape)

    def display(self):
        None

    def display_view(self, view, step):
        glClearColor(0.0, 0.0, 0.0, 0.0)
        light_pos = [ -0.0, 0.0, 200.0, 1.0 ]
        lf = view.look_from
        la = view.look_at
        self.initScene(light_pos, lf, la)
        rate=0
        if 0<view.step:
            rate=1.0*step/view.step
        self.putParts(view.part_list,rate)
        glFlush()
        glutSwapBuffers()
        glFinish()

    def initScene(self, light_pos, look_from, look_to):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()
        glLightfv(GL_LIGHT0, GL_POSITION, light_pos)
        gluLookAt(look_from[0], look_from[1], look_from[2], look_to[0], look_to[1], look_to[2], 0.0, 0.0, 1.0)

    def putParts(self, part_list,rate):
        for part in part_list:
            glScalef(1.0, 1.0, 1.0)
            glColor3f(0.5, 0.5, 0.5)
            glPushMatrix()
            pos=[0,0,0]
            pos[0]=part.position[0]+part.move[0]*rate
            pos[1]=part.position[1]+part.move[1]*rate
            pos[2]=part.position[2]+part.move[2]*rate
            glTranslatef(pos[0], pos[1], pos[2])
            glRotatef( part.rotation[2], 0.0, 0.0, 1.0) #yaw
            glRotatef( part.rotation[1], 0.0, 1.0, 0.0) #pitch
            glRotatef( part.rotation[0], 1.0, 0.0, 0.0) #roll

            glBegin(GL_TRIANGLES)
            for tri in part.triangles:
                length=tri.normal.length()
                glNormal3f(-tri.normal.x/length,-tri.normal.y/length,-tri.normal.z/length)
                glVertex([tri.points[0].x,tri.points[0].y,tri.points[0].z])
                glVertex([tri.points[1].x,tri.points[1].y,tri.points[1].z])
                glVertex([tri.points[2].x,tri.points[2].y,tri.points[2].z])
            glEnd()
            glPopMatrix()

    def capture(self):#name example: "output.png"
        width = glutGet(GLUT_WINDOW_WIDTH)
        height = glutGet(GLUT_WINDOW_HEIGHT)
        # capure
        glReadBuffer(GL_FRONT)
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1)
        data = glReadPixels(0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE)
        # sace image
        image = Image.frombytes("RGBA", (width, height), data)
        image = ImageOps.flip(image) # upside down
        return image


    def reshape(self, w, h):
        glViewport(0, 0, w, h)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(60, 1, 1.5, 2000)
        glMatrixMode(GL_MODELVIEW)

    # main process
    def timerProcess(self, arg):
        for view in self.target_view_list:
            if view.step == 0: # static image
                self.display_view(view, 0)
                img = self.capture()
                img.save(view.output_filepath, save_all=True)
                print("write image to ", view.output_filepath)
            else: # gif
                image_list=[]
                for i in range(view.step+1):
                    r = view.step - i
                    self.display_view(view, r)
                    img = self.capture()
                    image_list.append(img)
                append_images=image_list[1:]
                # pause for 1s
                for i in range(10):
                    append_images.append(image_list[-1])
                image_list[0].save(view.output_filepath, save_all=True, append_images=append_images, duration=100, loop=0)
                print("write gif to ", view.output_filepath)
        print("finish render")
        glutLeaveMainLoop()


if __name__ == '__main__':
    from sys import argv
    from roborecipe.StlLoader import *

    p1 = PartSource()
    p1.triangles = StlLoader("/home/ubuntu/roborecipe/sample/rr_base_plate.stl").get_triangles()
    p1.rotation = [0,-0,90]

    p2 = PartSource()
    p2.triangles = StlLoader("/home/ubuntu/roborecipe/sample/m3_spacer_20.stl").get_triangles()
    p2.position = [10,20,4]
    p2.rotation = [0,0,0]
    p2.move = [0,0,20]

    p3 = PartSource()
    p3.triangles = StlLoader("/home/ubuntu/roborecipe/sample/m3_spacer_20.stl").get_triangles()
    p3.position = [-10,20,4]
    p3.rotation = [0,0,0]
    p3.move = [0,0,20]

    p4 = PartSource()
    p4.triangles = StlLoader("/home/ubuntu/roborecipe/sample/rr_bar_plate.stl").get_triangles()
    p4.position = [0,20,24]
    p4.rotation = [0,0,90]
    p4.move = [0,0,30]

    view1 = ViewSource()
    view1.output_filepath = "/home/ubuntu/roborecipe/out.png"
    view1.part_list.append(p1)
    view1.part_list.append(p2)
    view1.part_list.append(p3)
    view1.part_list.append(p4)
    view1.look_from = [70,0,90]
    view1.look_at = [0,0,10]
    view1.step=0

    view2 = ViewSource()
    view2.output_filepath = "/home/ubuntu/roborecipe/out.gif"
    view2.part_list.append(p1)
    view2.part_list.append(p2)
    view2.part_list.append(p3)
    view2.part_list.append(p4)
    view2.look_from = [70,0,90]
    view2.look_at = [0,0,10]
    view2.step=10

    ig = ImageGenerator(sys.argv)
    # ig.renderViewList([view1, view2])
    ig.renderViewList([view2])



#!/usr/bin/python3

import os
from glob import glob
import xml.etree.ElementTree as ET
import pathlib
from roborecipe.data_type import *

class PackageParser:
	def __init__(self, path):
		self.path = path
		tree = ET.parse(str(pathlib.Path(path).resolve()))
		self.root = tree.getroot()
	def getName(self):
		return self.root.find("name").text
	def getDirPath(self):
		return str(pathlib.Path(self.path+"/..").resolve())

class PartConverter:
    def __init__(self):
        None

class ComponentParser:
	def __init__(self, pkg_path, component_path):
		pp = PackageParser(pkg_path)
		pkg_name = pp.getName()
		pkg_dir = pp.getDirPath()
		try:
			tree = ET.parse(str(pathlib.Path(component_path).resolve()))
		except:
			RED = '\033[31m'
			END = '\033[0m'
			print(RED + "parse error: " + component_path + END)
			return
		root = tree.getroot()

		if root.tag == 'part':
			self.component = self.ParsePart(pkg_name, pkg_dir, root)
		elif root.tag == 'assembly':
			self.component = self.ParseAssembly(pkg_name, root)
		else:
			self.component = DataComponent()

	def ParsePart(self, pkg_name, pkg_dir, root):
		component_name = root.find("name").text

		if root.find("stl_file") is not None:
			stl_file_name = pkg_dir+"/"+root.find("stl_file").text
		else:
			stl_file_name = ""


		if root.find("price") is not None:
			price = float(root.find("price").text)
		else:
			price = 0.0

		if root.find("distributor") is not None:
			distributor = root.find("distributor").text
		else:
			distributor = ''

		if root.find("description") is not None:
			description = root.find("description").text
		else:
			description = ''

		part = DataPart()
		part.id = ComponentIdentifier(pkg_name, component_name)
		part.price_value = price
		part.price_unit = "yen"
		part.distributor = distributor
		part.description = description
		part.stl_path = stl_file_name
		return part

	def ParseAssembly(self, pkg_name, root):
		component_name = root.find("name").text
		asm = DataAssembly()
		asm.id = ComponentIdentifier(pkg_name, component_name)
		for s in root.iter("step"):
			step = DataAssemblyStep()
			# parse components
			for c in s.iter("component"):
				child = DataAssemblyStepChild()
				child.id = ComponentIdentifier(c.attrib["pkg"], c.attrib["type"])
				child.transform = self.GetTransform(c)
				child.move = self.GetMove(c)
				step.child_list.append(child)
			# parse view
			for c in s.iter("view"):
				look_from = c.attrib["from"].split(' ')
				look_at = c.attrib["to"].split(' ')
				view = DataAssemblyView()
				view.look_at = [float(look_at[0]), float(look_at[1]), float(look_at[2])]
				view.look_from = [float(look_from[0]), float(look_from[1]), float(look_from[2])]
				step.view_list.append(view)

			asm.step_list.append(step)
		return asm

	def GetTransform(self, element):
		xyz = element.find("origin").attrib["xyz"].split(' ')
		rpy = element.find("origin").attrib["rpy"].split(' ')
		return Transform(float(xyz[0]), float(xyz[1]), float(xyz[2]), float(rpy[0]), float(rpy[1]), float(rpy[2]))

	def GetMove(self, element):
		xyz = element.find("move").attrib["xyz"].split(' ')
		return Point(float(xyz[0]), float(xyz[1]), float(xyz[2]))

class ComponentListParser:
	def __init__(self, path_pair_list):
		self.component_list = []
		for pp in path_pair_list:
			self.component_list.append(ComponentParser(pp[0], pp[1]).component)
	def getList(self):
		return self.component_list
			


if __name__ == '__main__':
	path_pair_list = [
		['../sample/srs007/package.xml', '../sample/srs007/main_asm/assemble.roborecipe'],
		['../sample/srs007/package.xml', '../sample/srs007/base_asm/assemble.roborecipe'],
		['../sample/pololu/package.xml', '../sample/pololu/4691_37mm_motor/description.roborecipe'],
		['../sample/iso_screw_m3/package.xml', '../sample/iso_screw_m3/pan_12/part.roborecipe']
	]
	for c in ComponentListParser(path_pair_list).getList():
		print(c.id.getName())




#!/usr/bin/python3

import os
from glob import glob
import xml.etree.ElementTree as ET
import pathlib
from roborecipe.data_type import *
from roborecipe.DirectoryLoader import *
import yaml

class ComponentListLoader:
	def __init__(self):
		self.component_list = []
		self.path_item_map = {}
	
	def load(self, path_item):
		with open(path_item.comp_file_path) as file:
			yaml_obj = yaml.safe_load(file)
			category_str = yaml_obj['category']
			if category_str == 'screw':
				self.component_list.append(ScrewData(path_item, yaml_obj))
			elif category_str == 'assembly':
				self.component_list.append(AssemblyData(path_item, yaml_obj))			
			elif category_str == 'laser_cut':
				self.component_list.append(LaserData(path_item, yaml_obj))			
			elif category_str == 'electric':
				self.component_list.append(ScrewData(path_item, yaml_obj)) # TODO
			elif category_str == 'mechanical':
				self.component_list.append(ScrewData(path_item, yaml_obj)) # TODO
			elif category_str == '3d_print':
				self.component_list.append(LaserData(path_item, yaml_obj)) # TODO
			elif category_str == 'order':
				self.component_list.append(LaserData(path_item, yaml_obj)) # TODO
			elif category_str == 'board':
				self.component_list.append(LaserData(path_item, yaml_obj)) # TODO
			elif category_str == 'harness':
				self.component_list.append(HarnessData(path_item, yaml_obj)) # TODO			
			else:
				RED = '\033[31m'
				END = '\033[0m'
				print(RED + 'unknown category: ' + category_str + END)
		self.path_item_map[path_item.pkg_name] = path_item
		
	def getList(self):
		return self.component_list

	def getComponent(self, id):
		for comp in self.component_list:
			if comp.id == id:
				return comp
		return None


class PackageParser:
	def __init__(self, path):
		self.path = path
		tree = ET.parse(str(pathlib.Path(path).resolve()))
		self.root = tree.getroot()
	def getName(self):
		return self.root.find("name").text
	def getDirPath(self):
		return str(pathlib.Path(self.path+"/..").resolve())

class PartConverter:
    def __init__(self):
        None

class ComponentLoader2:
	def __init__(self, pkg_path, component_path):
		pp = PackageParser(pkg_path)
		pkg_name = pp.getName()
		pkg_dir = pp.getDirPath()
		try:
			tree = ET.parse(str(pathlib.Path(component_path).resolve()))
		except:
			RED = '\033[31m'
			END = '\033[0m'
			print(RED + "parse error: " + component_path + END)
			return
		root = tree.getroot()

		if root.tag == 'part':
			self.component = self.ParsePart(pkg_name, pkg_dir, root)
		elif root.tag == 'assembly':
			self.component = self.ParseAssembly(pkg_name, root)
		else:
			self.component = DataComponent()

	def ParsePart(self, pkg_name, pkg_dir, root):
		component_name = root.find("name").text

		if root.find("stl_file") is not None:
			stl_file_name = pkg_dir+"/"+root.find("stl_file").text
		else:
			stl_file_name = ""


		if root.find("price") is not None:
			price = float(root.find("price").text)
		else:
			price = 0.0

		if root.find("distributor") is not None:
			distributor = root.find("distributor").text
		else:
			distributor = ''

		if root.find("description") is not None:
			description = root.find("description").text
		else:
			description = ''

		part = DataPart()
		part.id = ComponentIdentifier(pkg_name, component_name)
		part.price_value = price
		part.price_unit = "yen"
		part.distributor = distributor
		part.description = description
		part.stl_path = stl_file_name
		return part

	def ParseAssembly(self, pkg_name, root):
		component_name = root.find("name").text
		asm = DataAssembly()
		asm.id = ComponentIdentifier(pkg_name, component_name)
		for s in root.iter("step"):
			step = DataAssemblyStep()
			# parse components
			for c in s.iter("component"):
				child = DataAssemblyStepChild()
				child.id = ComponentIdentifier(c.attrib["pkg"], c.attrib["type"])
				child.transform = self.GetTransform(c)
				child.move = self.GetMove(c)
				step.child_list.append(child)
			# parse view
			for c in s.iter("view"):
				look_from = c.attrib["from"].split(' ')
				look_at = c.attrib["to"].split(' ')
				view = DataAssemblyView()
				view.look_at = [float(look_at[0]), float(look_at[1]), float(look_at[2])]
				view.look_from = [float(look_from[0]), float(look_from[1]), float(look_from[2])]
				step.view_list.append(view)

			asm.step_list.append(step)
		return asm

	def GetTransform(self, element):
		xyz = element.find("origin").attrib["xyz"].split(' ')
		rpy = element.find("origin").attrib["rpy"].split(' ')
		return Transform(float(xyz[0]), float(xyz[1]), float(xyz[2]), float(rpy[0]), float(rpy[1]), float(rpy[2]))

	def GetMove(self, element):
		xyz = element.find("move").attrib["xyz"].split(' ')
		return Point(float(xyz[0]), float(xyz[1]), float(xyz[2]))

class ComponentListParser:
	def __init__(self, path_pair_list):
		self.component_list = []
		for pp in path_pair_list:
			self.component_list.append(ComponentParser(pp[0], pp[1]).component)
	def getList(self):
		return self.component_list
			


if __name__ == '__main__':
	component_path_list = [
		ComponentPathItem('screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3/roborecipe/pan_10.yaml'),
		ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/side_asm.yaml'),
		ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/main_asm.yaml'),
	]

	cll = ComponentListLoader()
	for path_item in component_path_list:
		cll.load(path_item)
	for comp in cll.getList():
		print(comp.id.getName())
		if comp.category.value == ComponentCategoryEnum.ASSEMBLY:
			for step in comp.procedure_list:
				print('  step')
				for comp2 in step.component_list:
					print('    ' + comp2.id.getName())

	print('pkg_list')
	for pkg_item in cll.get_pkg_list():
		print(pkg_item)


#!/usr/bin/python3
from roborecipe.PartsPageGenerator import * 
from roborecipe.AssemblyAnalyzer import * 
from roborecipe.PartImageGenerator import * 


if __name__ == '__main__':
    component_path_list = [
        ComponentPathItem('screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3/roborecipe/pan_10.yaml'),
        ComponentPathItem('screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3/roborecipe/hollow_spacer_20.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/bar_plate.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/base_plate.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/side_asm.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/main_asm.yaml'),
    ]

    cll = ComponentListLoader()
    for path_item in component_path_list:
        cll.load(path_item)

    top_assembly_id = ComponentIdentifier('sample_project', 'main_asm')
    acl = AssemblyComponentListAnalyzer(cll, top_assembly_id)


    for item in acl.getQuantityList(ComponentCategoryEnum.SCREW):
        # image
        mpis = MechanicalPartsImageSource()
        mpis.mesh_file_path = item[0].rigit_body.mesh_filepath
        print(mpis.mesh_file_path)
        mpis.view_from = [item[0].view_from.x , item[0].view_from.y, item[0].view_from.z]
        mpis.view_to = [item[0].view_to.x , item[0].view_to.y, item[0].view_to.z]

        mpig = MechanicalPartsImageGenerator(mpis)
        mpig.save_image('test_image.png')

        # html
        mpd = MechanicalPartsPageData()
        mpd.title = item[0].id.getName() 
        mpd.overwiew_image = "test_image.png"
        mpd.description = item[0].description
        mpd.product_name = item[0].product.name
        mpd.product_url = item[0].product.url
        mpd.distributor_name = item[0].distributor_name
        mpd.distributor_url = item[0].distributor_url
        mpd.distributor_price = item[0].distributor_price


        mpg = MechanicalPartsPageGenerator(mpd)
        mpg.write('generate_index.html')
        break

        # name_body = item[0].id.getName()
        # name_link = item[0].id.getName() + '.html'
        # td.mechanical_purchased.add_line(ComponentTableLine(name_body,name_link,item[1],10,item[0].description))




#!/usr/bin/python3

import os
import sys
import pathlib
import argparse

# lib
from roborecipe.XmlParser import *
from roborecipe.TreeAnalyzer import *
from roborecipe.DirectorySearch import *
from roborecipe.HtmlGenerator import *
from roborecipe.ViewListGenerator import *

def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("command", help="sub command (list/generate)", type=str)
    parser.add_argument('option', nargs='*', default="")
    parser.add_argument("-d", "--directory", help="target directory")
    parser.add_argument("-o", "--output", help="output directory")
    parser.add_argument('-t', '--top_level_only', help='generate top level image only', action='store_true')
    args = parser.parse_args()

    target_directory = getTargetDirectory(args.directory)
    print('target directory: ' + str(target_directory))
    output_directory = getTargetDirectory(args.output)
    print('output directory: ' + str(output_directory))

    if args.command == "list":
        ds = DirectorySearch(target_directory)
        print("### package ###")
        for p in ds.getPackagePathList():
            print(p)
        print("### component ###")
        for c in ds.getComponentPathPairList():
            print(c[1])

    elif args.command == "generate":
        print("### generate ###")
        if len(args.option) != 2:
            print("tree must be 2 option")
            exit()
        pkg_name = args.option[0]
        type_name = args.option[1]
        generateInstruction(target_directory,output_directory, pkg_name, type_name, args.top_level_only)

    else:
        print("command error")

if __name__ == '__main__':
    main()



#!/usr/bin/python3

import os
import sys
import pathlib
import argparse

# lib
from roborecipe.XmlParser import *
from roborecipe.TreeAnalyzer import *
from roborecipe.DirectorySearch import *
from roborecipe.HtmlGenerator import *
from roborecipe.ViewListGenerator import *

# newlib
from roborecipe.DirectoryLoader import * 
from roborecipe.AssemblyAnalyzer import * 
from roborecipe.TopPageGenerator import * 
from roborecipe.PartsPageGenerator import *
from roborecipe.PartImageGenerator import *


def getTargetDirectory(target, default = ''):
    if target is None:
        output = pathlib.Path(default).resolve()
    else:
        output = pathlib.Path(str(target)).resolve()
    return output

def getCurrentDirectory():
    return str(pathlib.Path(__file__+'/..').resolve())

def output_list_data(target_directory):
    dl = DirectoryLoader(target_directory)
    component_path_list = []
    for component in dl.get_component_path_list(): 
        component_path_list.append(component)

    cll = ComponentListLoader()
    for path_item in component_path_list:
        cll.load(path_item)

    for comp in cll.getList():
        print(comp.id.getName())


def generateInstruction(target_directory, output_directory, pkg_name, type_name, top_level_only):
    # clear_panda3d_cashe()
    print(pkg_name, type_name)

    dl = DirectoryLoader(target_directory)
    out_ds = OutputDirectorySearch(output_directory)
    component_path_list = []
    for component in dl.get_component_path_list(): 
        component_path_list.append(component)

    cll = ComponentListLoader()
    for path_item in component_path_list:
        cll.load(path_item)

    top_assembly_id = ComponentIdentifier(pkg_name, type_name)
    ata = AssemblyTreeAnalyzer(cll, top_assembly_id)
    ada = AssemblyDependencyAnalyzer(cll, top_assembly_id)

    # top page
    tg = TopPageGenerator(top_assembly_id, ata, ada)
    tg.write(output_directory)

    # package copy
    used_pkg_name_set = ada.get_pkg_name_set()
    dl.copy_resouces_dir(used_pkg_name_set, output_directory)

    # part page (screw)
    for item in ata.get_quantity_list(ComponentCategoryEnum.SCREW):
        # image
        mpig = MechanicalPartsImageGenerator(item.comp)
        if out_ds.need_update_image(mpig.get_pkg_name(), mpig.get_m_datetime()):
            mpig.write(output_directory)
        else:
            print('skip image')

        # html
        mpg = MechanicalPartsPageGenerator(item.comp)
        mpg.write(output_directory)

    # part page (mechanical)
    for item in ata.get_quantity_list(ComponentCategoryEnum.MECHANICAL_ITEN):
        # image
        mpig = MechanicalPartsImageGenerator(item.comp)
        if out_ds.need_update_image(mpig.get_pkg_name(), mpig.get_m_datetime()):
            mpig.write(output_directory)
        else:
            print('skip image')

        # html
        mpg = MechanicalPartsPageGenerator(item.comp)
        mpg.write(output_directory)

    # part page (laser_cut)
    for item in ata.get_quantity_list(ComponentCategoryEnum.LASER_CUT):
        # image
        mpig = MechanicalPartsImageGenerator(item.comp)
        if out_ds.need_update_image(mpig.get_pkg_name(), mpig.get_m_datetime()):
            mpig.write(output_directory)
        else:
            print('skip image')

        # html
        mpg = ProcessPartsPageGenerator(item.comp)
        mpg.write(output_directory)

    # part page (3d_print)
    for item in ata.get_quantity_list(ComponentCategoryEnum.PRINT_3D):
        # image
        mpig = MechanicalPartsImageGenerator(item.comp)
        if out_ds.need_update_image(mpig.get_pkg_name(), mpig.get_m_datetime()):
            mpig.write(output_directory)
        else:
            print('skip image')

        # html
        mpg = ProcessPartsPageGenerator(item.comp)
        mpg.write(output_directory)

    # part page (order)
    for item in ata.get_quantity_list(ComponentCategoryEnum.ORDER):
        # image
        mpig = MechanicalPartsImageGenerator(item.comp)
        if out_ds.need_update_image(mpig.get_pkg_name(), mpig.get_m_datetime()):
            mpig.write(output_directory)
        else:
            print('skip image')

        # html
        mpg = ProcessPartsPageGenerator(item.comp)
        mpg.write(output_directory)

    # part page (electrical)
    for item in ata.get_quantity_list(ComponentCategoryEnum.ELECTRIC_ITEM):
        # image
        mpig = MechanicalPartsImageGenerator(item.comp)
        if out_ds.need_update_image(mpig.get_pkg_name(), mpig.get_m_datetime()):
            mpig.write(output_directory)
        else:
            print('skip image')

        # html
        mpg = MechanicalPartsPageGenerator(item.comp)
        mpg.write(output_directory)

    # part page (board)
    for item in ata.get_quantity_list(ComponentCategoryEnum.BOARD):
        # image
        mpig = MechanicalPartsImageGenerator(item.comp)
        if out_ds.need_update_image(mpig.get_pkg_name(), mpig.get_m_datetime()):
            mpig.write(output_directory)
        else:
            print('skip image')

        # html
        mpg = ProcessPartsPageGenerator(item.comp)
        mpg.write(output_directory)

    # part page (assembly)
    for item in ada.get_list():
        # image
        ata = AssemblyTreeAnalyzer(cll, item.comp.id)
        aig = AssemblyImageGenerator(item.comp, ata.get_position_list(), ata.get_step_view_list())
        asm_m_datetime = AssemblyDependencyAnalyzer(cll, item.comp.id).get_m_datetime()

        if out_ds.need_update_image(aig.get_pkg_name(), asm_m_datetime):
            aig.write(output_directory)
        else:
            print('skip image')

        # html
        apg = AssemblyPageGenerator(item.comp)
        apg.write(output_directory)


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("command", help="sub command (list/generate)", type=str)
    parser.add_argument('option', nargs='*', default="")
    parser.add_argument("-d", "--directory", help="target directory")
    parser.add_argument("-o", "--output", help="output directory")
    parser.add_argument('-t', '--top_level_only', help='generate top level image only', action='store_true')
    args = parser.parse_args()

    target_directory = getTargetDirectory(args.directory, '')
    print('target directory: ' + str(target_directory))
    output_directory = getTargetDirectory(args.output, 'out')
    print('output directory: ' + str(output_directory))

    if args.command == "list":
        output_list_data(target_directory)

    elif args.command == "generate":
        print("### generate ###")
        if len(args.option) != 2:
            print("tree must be 2 option")
            exit()
        pkg_name = args.option[0]
        type_name = args.option[1]
        generateInstruction(target_directory,output_directory, pkg_name, type_name, args.top_level_only)

    else:
        print("command error")

if __name__ == '__main__':
    main()



#!/usr/bin/python3

import math
from enum import Enum
import colorsys 

# common
class ComponentIdentifier:
	def __init__(self, pkg = "unknown", type = "unknown"):
		self.pkg_name = pkg
		self.type_name = type
	def getName(self):
		return self.pkg_name + "/" + self.type_name
	def __eq__(self, other):
		if not isinstance(other, ComponentIdentifier):
			return NotImplemented
		return self.pkg_name == other.pkg_name and self.type_name == other.type_name    
	def __hash__(self):
		return hash((self.pkg_name, self.type_name))

class ComponentCategoryEnum(Enum):
	UNKNOWN = 0
	# to make instruction steps
	ASSEMBLY = 1
	# harnes_connection related
	HARNESS = 2
	BOARD = 3
	ELECTRIC_ITEM = 4
	# rigit body related
	MECHANICAL_ITEN = 5
	SCREW = 6
	PRINT_3D = 7
	LASER_CUT = 8
	ORDER = 9

class ComponentCategory:
	def __init__(self, category_str):
		if category_str == 'assembly':
			self.value = ComponentCategoryEnum.ASSEMBLY
		elif category_str == 'screw':
			self.value = ComponentCategoryEnum.SCREW
		elif category_str == 'mechanical':
			self.value = ComponentCategoryEnum.MECHANICAL_ITEN
		elif category_str == 'laser_cut':
			self.value = ComponentCategoryEnum.LASER_CUT
		elif category_str == 'electric':
			self.value = ComponentCategoryEnum.ELECTRIC_ITEM
		elif category_str == '3d_print':
			self.value = ComponentCategoryEnum.PRINT_3D
		elif category_str == 'order':
			self.value = ComponentCategoryEnum.ORDER
		elif category_str == 'board':
			self.value = ComponentCategoryEnum.BOARD
		elif category_str == 'harness':
			self.value = ComponentCategoryEnum.HARNESS
		else:
			self.value = ComponentCategoryEnum.UNKNOWN

	def is_assembly(self):
		return (self.value == ComponentCategoryEnum.ASSEMBLY)

	def get_color(self, focused):
		if focused:
			s = 1.0
			v = 1.0
		else:
			s = 0.5
			v = 0.5

		if self.value == ComponentCategoryEnum.SCREW:
			h = 0.66
		elif self.value == ComponentCategoryEnum.LASER_CUT:
			h = 0.1
		elif self.value == ComponentCategoryEnum.ELECTRIC_ITEM:
			h = 0.3
		elif self.value == ComponentCategoryEnum.PRINT_3D:
			h = 0.4
		else:
			h = 0.0
			s = 0.0

		rgb = colorsys.hsv_to_rgb(h, s, v)
		return (rgb[0], rgb[1], rgb[2], 1.0)

class ComponentCommon:
	def __init__(self, pkg_base_path, yaml_obj):
		self.description = yaml_obj['description']
		self.image_list = []
		if 'image_list' in yaml_obj:
			self.image_list = yaml_obj['image_list']

class RigitBody:
	def __init__(self, pkg_base_path, yaml_obj):
		self.mesh_filepath = pkg_base_path + '/' + yaml_obj['mesh']['path']
		self.view = View(yaml_obj['view'])

class View:
	def __init__(self, yaml_obj):
		from_obj = yaml_obj['from']
		self.from_point = Point(from_obj['x'], from_obj['y'], from_obj['z'])
		to_obj = yaml_obj['to']
		self.to_point = Point(to_obj['x'], to_obj['y'], to_obj['z'])
	def __repr__(self):
		return 'view(' + str(self.from_point) + '->' + str(self.to_point) + ')'

class Product:
	def __init__(self, yaml_obj):
		self.name = yaml_obj['name']
		self.url = yaml_obj['url']

class Distributor:
	def __init__(self, yaml_obj):
		self.name = yaml_obj['name']
		self.url = yaml_obj['url']
		self.price = yaml_obj['price']

class Process:
	def __init__(self, pkg_base_path, yaml_obj):
		self.data_filepath = pkg_base_path + '/' + yaml_obj['data']['path']
		self.material = yaml_obj['material']
		self.cost = yaml_obj['cost']	

class ScrewData:
	def __init__(self, path_item, yaml_obj):
		self.id = ComponentIdentifier(path_item.pkg_name, yaml_obj['name'])
		self.category = ComponentCategory(yaml_obj['category'])
		self.pkg_base_path = path_item.pkg_base_path
		self.m_datetime = path_item.m_datetime
		# common
		self.common = ComponentCommon(path_item.pkg_base_path, yaml_obj['common'])
		# body related
		self.rigit_body = RigitBody(path_item.pkg_base_path, yaml_obj['rigit_body'])
		# product
		self.product = Product(yaml_obj['product'])
		# distributor
		self.distributor_list = []
		for yaml_item in yaml_obj['distributor']:
			self.distributor_list.append(Distributor(yaml_item))

class LaserData:
	def __init__(self, path_item, yaml_obj):
		self.id = ComponentIdentifier(path_item.pkg_name, yaml_obj['name'])
		self.category = ComponentCategory(yaml_obj['category'])
		self.pkg_base_path = path_item.pkg_base_path
		self.m_datetime = path_item.m_datetime

		self.common = ComponentCommon(path_item.pkg_base_path, yaml_obj['common'])
		self.rigit_body = RigitBody(path_item.pkg_base_path, yaml_obj['rigit_body'])
		self.process = Process(path_item.pkg_base_path, yaml_obj['process'])

class Harness:
	def __init__(self, pkg_base_path, yaml_obj):
		self.cost = yaml_obj['cost']
		self.line_kind = yaml_obj['line']['kind']
		self.line_length = yaml_obj['line']['length']
		self.line_name_list = yaml_obj['line']['names']
		self.end1 = HarnessEnd(yaml_obj['end1'])
		self.end2 = HarnessEnd(yaml_obj['end2'])

class HarnessEnd:
	def __init__(self, yaml_obj):
		self.family = yaml_obj['family']
		self.model = yaml_obj['model']
		self.assign_list = yaml_obj['assign']

class HarnessData:
	def __init__(self, path_item, yaml_obj):
		self.id = ComponentIdentifier(path_item.pkg_name, yaml_obj['name'])
		self.category = ComponentCategory(yaml_obj['category'])
		self.pkg_base_path = path_item.pkg_base_path
		self.m_datetime = path_item.m_datetime

		self.common = ComponentCommon(path_item.pkg_base_path, yaml_obj['common'])
		self.harness = Harness(path_item.pkg_base_path, yaml_obj['harness'])

class AssemblyData:
	def __init__(self, path_item, yaml_obj):
		self.id = ComponentIdentifier(path_item.pkg_name, yaml_obj['name'])
		self.category = ComponentCategory(yaml_obj['category'])
		self.pkg_base_path = path_item.pkg_base_path
		self.m_datetime = path_item.m_datetime

		self.common = ComponentCommon(path_item.pkg_base_path, yaml_obj['common'])
		self.procedure_list = []
		for step in yaml_obj['procedure_list']:
			self.procedure_list.append(AssemblyStepData(step))

class AssemblyStepData:
	def __init__(self, yaml_obj):
		self.component_list = []
		for comp in yaml_obj['component_list']:
			self.component_list.append(AssemblyComponentData(comp))
		self.view_list = []
		for view in yaml_obj['view_list']:
			self.view_list.append(View(view))

class AssemblyComponentData:
	def __init__(self, yaml_obj):
		self.id  = ComponentIdentifier(yaml_obj['pkg'], yaml_obj['type'])
		ori = yaml_obj['origin']
		self.transform = Transform(ori['x'], ori['y'], ori['z'], ori['roll'], ori['pitch'], ori['yaw'])
		self.move = Point(yaml_obj['move']['x'], yaml_obj['move']['y'], yaml_obj['move']['z'])

# transform
class Point:
	def __init__(self, x=0.0, y=0.0, z=0.0):
		self.x = x
		self.y = y
		self.z = z
	def __add__(self, other):
		p =Point()
		p.x = self.x + other.x
		p.y = self.y + other.y
		p.z = self.z + other.z
		return p

	def __repr__(self) -> str:
		return "("+str(self.x)+","+str(self.y)+","+str(self.z)+")"

class Quaternion:
	def __init__(self, w=1.0, x=0.0, y=0.0, z=0.0): # degree
		self.w = w
		self.x = x
		self.y = y
		self.z = z

	def __mul__(self, other): # TODO
		output = Quaternion()
		output.w = self.w * other.w - self.x * other.x - self.y * other.y - self.z * other.z
		output.x = self.w * other.x + self.x * other.w + self.y * other.z - self.z * other.y
		output.y = self.w * other.y + self.y * other.w + self.z * other.x - self.x * other.z
		output.z = self.w * other.z + self.z * other.w + self.x * other.y - self.y * other.x
		return output
		# self_rpy = self.getRPY()
		# other_rpy = other.getRPY()
		# output = Quaternion().setRPY(self_rpy[0]+other_rpy[0],self_rpy[1]+other_rpy[1],self_rpy[2]+other_rpy[2])
		# return output

	def __repr__(self) -> str:
		(roll, pitch, yaw) = self.getRPY()
		return "q("+ str(self.w) + ',' + str(self.x) + ',' + str(self.y) + ',' + str(self.z) +") rpy("+str(roll)+","+str(pitch)+","+str(yaw)+")"
		# return "("+str(self.roll)+","+str(self.pitch)+","+str(self.yaw)+")"

	def setRPY(self, roll, pitch, yaw): # degree
		cy = math.cos(yaw*math.pi/180 * 0.5)
		sy = math.sin(yaw*math.pi/180 * 0.5)
		cp = math.cos(pitch*math.pi/180 * 0.5)
		sp = math.sin(pitch*math.pi/180 * 0.5)
		cr = math.cos(roll*math.pi/180 * 0.5)
		sr = math.sin(roll*math.pi/180 * 0.5)
		self.w = cr * cp * cy + sr * sp * sy
		self.x = sr * cp * cy - cr * sp * sy
		self.y = cr * sp * cy + sr * cp * sy
		self.z = cr * cp * sy - sr * sp * cy
		return self

	def getRPY(self): # degree
		q0q0 = self.w * self.w
		q1q1 = self.x * self.x
		q2q2 = self.y * self.y
		q3q3 = self.z * self.z
		q0q1 = self.w * self.x
		q0q2 = self.w * self.y
		q0q3 = self.w * self.z
		q1q2 = self.x * self.y
		q1q3 = self.x * self.z
		q2q3 = self.y * self.z
		roll = math.atan2((2.0 * (q2q3 + q0q1)), (q0q0 - q1q1 - q2q2 + q3q3))
		pitch = -math.asin((2.0 * (q1q3 - q0q2)))
		yaw = math.atan2((2.0 * (q1q2 + q0q3)), (q0q0 + q1q1 - q2q2 - q3q3))
		return [roll*180/math.pi, pitch*180/math.pi, yaw*180/math.pi]

	def setPoint(self, position):
		self.w = 0
		self.x = position.x
		self.y = position.y
		self.z = position.z
		return self

	def getPoint(self):
		return Point(self.x, self.y, self.z)

	def getConj(self):
		return Quaternion(self.w,-self.x,-self.y,-self.z)

class Transform:
	def __init__(self, x=0.0, y=0.0, z=0.0, roll=0, pitch=0, yaw=0):
		self.position = Point(x, y, z)
		self.rotation = Quaternion().setRPY(roll, pitch, yaw)

	def __mul__(self, other):
		output = Transform()
		pos_q = Quaternion().setPoint(other.position)
		conv_pos_q = self.rotation * pos_q * self.rotation.getConj()
		conv_pos = conv_pos_q.getPoint()
		output.position = self.position + conv_pos
		output.rotation = self.rotation * other.rotation
		return output

	def __repr__(self) -> str:
		return "tf " + str(self.position) + " " + str(self.rotation)

# component internal data
class DataComponent:
	def __init__(self, pkg="unknown", type="unknown", pkg_path="", cmp_path=""):
		self.initial_char = "C"
		self.id = ComponentIdentifier(pkg, type)
		self.pkg_path = pkg_path
		self.cmp_path = cmp_path

class DataPart(DataComponent):
	def __init__(self, pkg="unknown", type="unknown", pkg_path="", cmp_path=""):
		super().__init__(pkg, type, pkg_path, cmp_path)
		self.initial_char = "P"
		self.price_value = 0
		self.price_unit = 'yen'
		self.distributor = ''
		self.description = ''
		self.stl_path = ''

class DataAssemblyStepChild:
	def __init__(self):
		self.id = ComponentIdentifier()
		self.transform = Transform()
		self.move = Point()

class DataAssemblyView:
	def __init__(self):
		self.look_at = [0,0,0]
		self.look_from = [0,0,0]

class DataAssemblyStep:
	def __init__(self):
		self.view_list = []
		self.child_list = []

class DataAssembly(DataComponent):
	def __init__(self, pkg="unknown", type="unknown", pkg_path="", cmp_path=""):
		super().__init__(pkg, type, pkg_path, cmp_path)
		self.initial_char = "A"
		self.step_list = []

if __name__ == '__main__':
	t1 = Transform()
	t1.position.x = 0
	t1.position.y = 0
	t1.position.z = 4
	t1.rotation.setRPY(0,0,-30)

	t2 = Transform()
	t2.position.x = 0
	t2.position.y = 0
	t2.position.z = 4
	t2.rotation.setRPY(180,0,0)

	print(t1*t2)


from panda3d_viewer import Viewer, ViewerConfig
from roborecipe.data_type import *
import imageio
import datetime
import os
import pathlib
import shutil
from panda3d.core import BamCache

def clear_panda3d_cashe():
    cashe_dir = BamCache.getGlobalPtr().getRoot()
    print("remove panda3d chashe " + str(cashe_dir))
    shutil.rmtree(cashe_dir)

class MechanicalPartsImageGenerator:
    def __init__(self, comp):
        self.comp = comp

    def get_pkg_name(self):
        return self.comp.id.pkg_name

    def get_m_datetime(self):
        return self.comp.m_datetime

    def write(self, output_base_dir):
        config = ViewerConfig()
        config.set_window_size(800, 800)
        config.enable_shadow(True)
        config.show_axes(False)
        config.show_grid(False)

        viewer = Viewer(window_type='offscreen', config=config)
        viewer.append_group('root')

        mm_to_unit = 0.01
        viewer.append_mesh('root', 'mesh1', self.comp.rigit_body.mesh_filepath, scale={mm_to_unit, mm_to_unit, mm_to_unit})
        viewer.set_material('root', 'mesh1', color_rgba=(0.5, 0.5, 0.0, 1))
 
        viewer.move_nodes('root', {'mesh1': ((0, 0, 0), (1, 0, 0, 0))})

        body_view = self.comp.rigit_body.view
        point_from = (body_view.from_point.x * mm_to_unit, body_view.from_point.y * mm_to_unit, body_view.from_point.z * mm_to_unit)
        point_to = (body_view.to_point.x * mm_to_unit, body_view.to_point.y * mm_to_unit, body_view.to_point.z * mm_to_unit)

        viewer.reset_camera(pos=point_from, look_at=point_to)
        image = viewer.get_screenshot(requested_format='RGB')
        viewer.destroy()

        output_dir = str(output_base_dir) + '/' + self.comp.id.pkg_name + '/images'
        os.makedirs(output_dir, exist_ok = True)

        filepath = output_dir + '/' + self.comp.id.type_name + '_rigid_body.png'
        print('write ' + filepath)
        writer = imageio.get_writer(filepath, mode='I')
        writer.append_data(image)

class AssemblyImageGenerator:
    def __init__(self, comp, item_list, step_view_list):
        self.comp = comp
        self.item_list = item_list
        self.step_view_list = step_view_list

    def get_pkg_name(self):
        return self.comp.id.pkg_name

    def write(self, output_base_dir):
        config = ViewerConfig()
        config.set_window_size(800, 800)
        config.enable_shadow(True)
        config.show_axes(False)
        config.show_grid(False)
        viewer = Viewer(window_type='offscreen', config=config)
        viewer.enable_lights(False)

        #### change lights ####
        viewer._app._lights = [
            viewer._app._make_light_ambient((0.2, 0.2, 0.2)),
            viewer._app._make_light_direct(
                1, (0.3, 0.4, 0.4), pos=(8.0, 8.0, 10.0)),
            viewer._app._make_light_direct(
                2, (0.4, 0.3, 0.4), pos=(8.0, -8.0, 10.0)),
            viewer._app._make_light_direct(
                3, (0.4, 0.4, 0.3), pos=(-8.0, 8.0, 10.0)),
            viewer._app._make_light_direct(
                4, (0.3, 0.3, 0.4), pos=(-8.0, -8.0, 10.0)),
            viewer._app._make_light_direct(
                5, (0.8, 0.8, 0.8), pos=(0.0, 0.0, -10.0)),
        ]
        viewer._app._lights_mask = [True, True, True, True, True, True]
        viewer.enable_lights(True)
        #### end change lights ####

        viewer.append_group('root')
        mm_to_unit = 0.01
        max_step = self._get_max_step()

        self.image_list = []
        for step_no in range(max_step + 1):
            # add object
            for obj_no, item in enumerate(self.item_list):
                mesh_name = 'mesh' + str(obj_no)
                if item.step_no == step_no:
                    # print(mesh_name, item.comp.rigit_body.mesh_filepath)
                    viewer.append_mesh('root', mesh_name, item.comp.rigit_body.mesh_filepath, scale={mm_to_unit, mm_to_unit, mm_to_unit})

            #  set color
            for obj_no, item in enumerate(self.item_list):
                mesh_name = 'mesh' + str(obj_no)
                if item.step_no == step_no:
                    color_rgba = item.comp.category.get_color(True)
                    viewer.set_material('root', mesh_name, color_rgba=color_rgba)
                elif item.step_no < step_no:
                    color_rgba = item.comp.category.get_color(False)
                    viewer.set_material('root', mesh_name, color_rgba=color_rgba)

            # view
            for view_no, view in enumerate(self.step_view_list[step_no]):
                point_from = (view.from_point.x * mm_to_unit, view.from_point.y * mm_to_unit, view.from_point.z * mm_to_unit)
                point_to = (view.to_point.x * mm_to_unit, view.to_point.y * mm_to_unit, view.to_point.z * mm_to_unit)
                viewer.reset_camera(pos=point_from, look_at=point_to)

                image_list = []
                # set position & render
                move_size = 10
                for move_index in range(move_size + 1):
                    rate = 1.0 * (move_size - move_index) / move_size  
                    for obj_no, item in enumerate(self.item_list):
                        if item.step_no == step_no:
                            mesh_name = 'mesh' + str(obj_no)
                            pos = item.transform.position
                            rot = item.transform.rotation
                            move = item.move
                            pos_unit = ((pos.x + move.x * rate) * mm_to_unit, (pos.y + move.y * rate) * mm_to_unit, (pos.z + move.z * rate) * mm_to_unit)
                            rot_unit = (rot.w, rot.x, rot.y, rot.z)
                            viewer.move_nodes('root', {mesh_name: (pos_unit, rot_unit)})
                            # print(pos_unit, rot_unit)
                    image_list.append(viewer.get_screenshot(requested_format='RGB'))
                self.image_list.append([image_list, step_no, view_no])
        viewer.stop()

        image_dir = str(output_base_dir) + '/' + self.comp.id.pkg_name + '/images/'
        os.makedirs(image_dir, exist_ok = True)

        for image_set in self.image_list:
            image_list = image_set[0]
            step_no = image_set[1]
            view_no = image_set[2]

            gif_name = self.comp.id.type_name + '_' + str(step_no) + '_' + str(view_no) + '.gif'
            print('write image ' + image_dir + gif_name)

            last_image = image_list[-1].copy()
            for i in range(10):
                image_list.append(last_image)
            imageio.mimsave(image_dir + gif_name, image_list, duration = 200, loop = 0)

    def _get_max_step(self):
        def key_func(obj):
            return obj.step_no
        return max(self.item_list, key = key_func).step_no


if __name__ == '__main__':
    from roborecipe.AssemblyAnalyzer import *
    import os

    component_path_list = [
        ComponentPathItem('screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3/roborecipe/pan_10.yaml'),
        ComponentPathItem('screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3/roborecipe/hollow_spacer_20.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/bar_plate.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/base_plate.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/side_asm.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/main_asm.yaml'),
    ]
    cll = ComponentListLoader()
    for path_item in component_path_list:
        cll.load(path_item)



    # assembly images
    top_assmebly_id = ComponentIdentifier('sample_project', 'main_asm')

    ata = AssemblyTreeAnalyzer(cll, top_assmebly_id)
    for item in ata.get_position_list():
        print(item.comp.rigit_body.mesh_filepath)

    aig = AssemblyImageGenerator(ata.get_position_list(), ata.get_step_view_list())

    os.makedirs('../out/sample_project/images', exist_ok = True)
    aig.save_image('../out/sample_project/images/main_asm')



import struct
import math
import os

from OpenGL.GL import *
from OpenGL.GLU import *

#class for a 3d point
class createpoint:
    def __init__(self,p,c=(1,0,0)):
        self.point_size=0.5
        self.color=c
        self.x=p[0]
        self.y=p[1]
        self.z=p[2]
      
    def glvertex(self):
        glVertex3f(self.x,self.y,self.z)

    def length(self):
        return math.sqrt(self.x*self.x+self.y*self.y+self.z*self.z)

#class for a 3d face on a model
class createtriangle:
    points=None
    normal=None

    def __init__(self,p1,p2,p3,n=None):
        #3 points of the triangle
        self.points=createpoint(p1),createpoint(p2),createpoint(p3)
      
        #triangles normal
        self.normal=createpoint(self.calculate_normal(self.points[0],self.points[1],self.points[2]))#(0,1,0)#
  
    #calculate vector / edge
    def calculate_vector(self,p1,p2):
        return -p1.x+p2.x,-p1.y+p2.y,-p1.z+p2.z
      
    def calculate_normal(self,p1,p2,p3):
        a=self.calculate_vector(p3,p2)
        b=self.calculate_vector(p3,p1)
        #calculate the cross product returns a vector
        return self.cross_product(a,b)    
  
    def cross_product(self,p1,p2):
        return (p1[1]*p2[2]-p2[1]*p1[2]) , (p1[2]*p2[0])-(p2[2]*p1[0]) , (p1[0]*p2[1])-(p2[0]*p1[1])

class StlLoader:
    #load stl file detects if the file is a text file or binary file
    def __init__(self, filename):
        self.model=[]
        fp=open(filename,'rb')
        h=fp.read(80)
        type=h[0:5]
        fp.close()

        if type=='solid':
            # print("reading text file"+str(filename))
            self.load_text_stl(filename)
        else:
            # print("reading binary stl file "+str(filename))
            self.load_binary_stl(filename)

    #return the faces of the triangles
    def get_triangles(self):
        return self.model

    def get_triangles_generator(self):
        if self.model:
            for face in self.model:
                yield face

    #draw the models faces
    def draw(self):
        glBegin(GL_TRIANGLES)
        for tri in self.get_triangles():
            length=tri.normal.length()
            glNormal3f(-tri.normal.x/length,-tri.normal.y/length,-tri.normal.z/length)
            glVertex([tri.points[0].x,tri.points[0].y,tri.points[0].z])
            glVertex([tri.points[1].x,tri.points[1].y,tri.points[1].z])
            glVertex([tri.points[2].x,tri.points[2].y,tri.points[2].z])
        glEnd()
  
    #read text stl match keywords to grab the points to build the model
    def load_text_stl(self,filename):
        fp=open(filename,'r')

        for line in fp.readlines():
            words=line.split()
            if len(words)>0:
                if words[0]=='solid':
                    self.name=words[1]

                if words[0]=='facet':
                    center=[0.0,0.0,0.0]
                    triangle=[]
                    normal=(eval(words[2]),eval(words[3]),eval(words[4]))
                  
                if words[0]=='vertex':
                    triangle.append((eval(words[1]),eval(words[2]),eval(words[3])))
                  
                  
                if words[0]=='endloop':
                    #make sure we got the correct number of values before storing
                    if len(triangle)==3:
                        self.model.append(createtriangle(triangle[0],triangle[1],triangle[2],normal))
        fp.close()

    #load binary stl file check wikipedia for the binary layout of the file
    #we use the struct library to read in and convert binary data into a format we can use
    def load_binary_stl(self,filename):
        fp=open(filename,'rb')
        h=fp.read(80)

        l=struct.unpack('I',fp.read(4))[0]
        count=0
        while True:
            try:
                p=fp.read(12)
                if len(p)==12:
                    n=struct.unpack('f',p[0:4])[0],struct.unpack('f',p[4:8])[0],struct.unpack('f',p[8:12])[0]
                  
                p=fp.read(12)
                if len(p)==12:
                    p1=struct.unpack('f',p[0:4])[0],struct.unpack('f',p[4:8])[0],struct.unpack('f',p[8:12])[0]

                p=fp.read(12)
                if len(p)==12:
                    p2=struct.unpack('f',p[0:4])[0],struct.unpack('f',p[4:8])[0],struct.unpack('f',p[8:12])[0]

                p=fp.read(12)
                if len(p)==12:
                    p3=struct.unpack('f',p[0:4])[0],struct.unpack('f',p[4:8])[0],struct.unpack('f',p[8:12])[0]

                new_tri=(n,p1,p2,p3)

                if len(new_tri)==4:
                    tri=createtriangle(p1,p2,p3,n)
                    self.model.append(tri)
                count+=1
                fp.read(2)

                if len(p)==0:
                    break
            except EOFError:
                break
        fp.close()

def model_dict_load(model_dict):
	stl_dict={}
	for model in model_dict:
		if not model in stl_dict: 
			imodel=stl_load()
			print("test",model,model_dict[model])
			imodel.load_stl(os.path.abspath('')+"/"+model_dict[model])
			stl_dict[model]=imodel
	return stl_dict

if __name__ == '__main__':
    print("start")
    stl1 = stl_load()
    stl1.load_stl("/home/ubuntu/roborecipe/old_resources/Laser1.stl")
    stl1.draw()


#!/usr/bin/python3
import pathlib
import glob

class DirectorySearch:
    def __init__(self, root_path):
        target_directory = pathlib.Path(root_path).resolve()
        self.pkg_list = self._get_package_list(target_directory)

        self.component_list = []
        for p in self.pkg_list:
            for c in self._get_component_list(p):
                self.component_list.append([p,c])

    def getComponentPathPairList(self):
        return self.component_list

    def getPackagePathList(self):
        return self.pkg_list

    def _get_package_list(self, target_directory):
        return glob.glob(str(target_directory) + '/*/package.xml', recursive = True)

    def _get_component_list(self, pkg_path):
        pkg_dir = pathlib.Path(pkg_path+"/..").resolve()
        return glob.glob(str(pkg_dir) + '/*/*.roborecipe', recursive = True)

if __name__ == '__main__':
    ds = DirectorySearch('../sample')
    for component in ds.getComponentPathPairList(): 
        print(component)



#!/usr/bin/python3
from jinja2 import Template
import os
import pathlib

from roborecipe.data_type import * 

class ComponentTableLine:
    def __init__(self, name_body = "", name_link = "", quantity = 0, price = 0, description = ""):
        self.name_body = name_body
        self.name_link = name_link
        self.quantity = quantity
        self.price = price
        self.description = description

class ComponentTable:
    def __init__(self):
        self.list = []
        self.total_price = 0
    def add_line(self, line):
        self.list.append(line)
        self.total_price += line.price * line.quantity

class TopPageData:
    def __init__(self):
        self.title = ""
        self.mechanical_purchased = ComponentTable()
        self.mechanical_fabrication = ComponentTable()
        self.electric_purchased = ComponentTable()
        self.electric_fabrication = ComponentTable()
        self.assembly = ComponentTable()

class TopPageGenerator:
    def __init__(self, comp_id, tree_analyer, dependency_analyzer):
        self.comp_id = comp_id
        self.tree_analyer = tree_analyer
        self.dependency_analyzer = dependency_analyzer

        template_filepth = pathlib.Path(__file__).parent / 'templates' / 'top.html'
        file = open(template_filepth)
        self.template = Template(file.read())

    def write(self, output_base_dir):
        td = TopPageData()
        td.title = self.comp_id.getName()

        mechanical_list = []
        mechanical_list.extend(self.tree_analyer.get_quantity_list(ComponentCategoryEnum.SCREW))
        mechanical_list.extend(self.tree_analyer.get_quantity_list(ComponentCategoryEnum.MECHANICAL_ITEN))
        for item in mechanical_list:
            name_body = item.comp.id.getName()
            name_link = item.comp.id.getName() + '.html'
            price = item.comp.distributor_list[0].price
            td.mechanical_purchased.add_line(ComponentTableLine(name_body,name_link,item.quantity,price,item.comp.common.description))

        fablication_list = []
        fablication_list.extend(self.tree_analyer.get_quantity_list(ComponentCategoryEnum.LASER_CUT))
        fablication_list.extend(self.tree_analyer.get_quantity_list(ComponentCategoryEnum.PRINT_3D))
        fablication_list.extend(self.tree_analyer.get_quantity_list(ComponentCategoryEnum.ORDER))
        for item in fablication_list:
            name_body = item.comp.id.getName()
            name_link = item.comp.id.getName() + '.html'
            price = item.comp.process.cost
            td.mechanical_fabrication.add_line(ComponentTableLine(name_body,name_link,item.quantity,price,item.comp.common.description))

        for item in self.tree_analyer.get_quantity_list(ComponentCategoryEnum.ELECTRIC_ITEM):
            name_body = item.comp.id.getName()
            name_link = item.comp.id.getName() + '.html'
            price = item.comp.distributor_list[0].price
            td.electric_purchased.add_line(ComponentTableLine(name_body,name_link,item.quantity,price,item.comp.common.description))

        for item in self.tree_analyer.get_quantity_list(ComponentCategoryEnum.BOARD):
            name_body = item.comp.id.getName()
            name_link = item.comp.id.getName() + '.html'
            price = item.comp.process.cost
            td.electric_fabrication.add_line(ComponentTableLine(name_body,name_link,item.quantity,price,item.comp.common.description))

        for item in self.dependency_analyzer.get_list():
            name_body = item.comp.id.getName()
            name_link = item.comp.id.getName() + '.html'
            td.assembly.add_line(ComponentTableLine(name_body,name_link,item.quantity,0,item.comp.common.description))

        rendered_str = self.template.render(data = td)

        os.makedirs(output_base_dir, exist_ok = True)
        output_file = str(output_base_dir) + '/index.html'
        with open(output_file, mode='w') as f:
            f.write(rendered_str)
        print('write html ' + output_file)

if __name__ == '__main__':
    td = TopPageData()
    tg = TopPageGenerator(td)
    tg.write('generate_index.html')


#!/usr/bin/python3
from jinja2 import Template
from roborecipe.data_type import * 

class HtmlParchageItem:
    def __init__(self):
        self.id = ComponentIdentifier()
        self.quantity = 0
        self.price_value = 0
        self.price_unit = 'yen'
        self.distributor = 'http:/aaa.bbb'
        self.description = 'description'

class HtmlComponentItem:
    def __init__(self, pkg = "unknown", type = "unknown", quantity = 0):
        self.id = ComponentIdentifier(pkg, type)
        self.quantity = quantity
    def increaseQuantity(self):
        self.quantity += 1

class HtmlAssemblyStep:
    def __init__(self):
        self.seq_no = 0
        self.component_list = []
        self.image_path_list = []

class HtmlAssemblyItem:
    def __init__(self):
        self.name = "unknown"
        self.quantity = 0
        self.step_list = []

class Cost:
    def __init__(self):
        self.value = 0
        self.uni = "yen"

class HtmlGenerator:
    def __init__(self):
        self.template_path = ""
        self.title = "roborecipe"
        self.part_list = []
        self.assembly_list = []
        self.total_cost = Cost()
    def generate(self, output_path):
        with open(self.template_path) as f:
            template = Template(f.read())
            ren_s = template.render(tittle=self.title, part_list=self.part_list, assembly_list=self.assembly_list, total_cost=self.total_cost)

        with open(output_path, mode='w') as f:
            f.write(ren_s)
        print('write ' + output_path)

if __name__ == '__main__':
    html_generator = HtmlGenerator()
    html_generator.template_path = 'templates/index.html'

    p1 = HtmlParchageItem()
    p1.id = ComponentIdentifier('p1', 't1')
    p1.quantity = 1
    p2 = HtmlParchageItem()
    p2.id = ComponentIdentifier('p2', 't2')
    p2.quantity = 2
    html_generator.part_list = [p1, p2]



    a1 = HtmlAssemblyItem()
    a1.name = "A1"
    a1.quantity = 1
    a1_1 = HtmlAssemblyStep()
    a1_1.seq_no = 1
    a1_1.component_list.append(HtmlComponentItem("p1", "t1", 1))
    a1_1.component_list.append(HtmlComponentItem("p2", "t2", 2))
    a1_1.image_path_list.append("../image/assemble_0_0.gif")
    a1.step_list.append(a1_1)

    a1_2 = HtmlAssemblyStep()
    a1_2.seq_no = 2
    a1_2.component_list.append(HtmlComponentItem("p1", "t1", 1))
    a1_2.component_list.append(HtmlComponentItem("p2", "t2", 2))
    a1_2.image_path_list.append("../image/assemble_1_0.gif")
    a1_2.image_path_list.append("../image/assemble_1_1.gif")
    a1.step_list.append(a1_2)

    a2 = HtmlAssemblyItem()
    a2.name = "A2"
    a2.quantity = 2

    html_generator.assembly_list =[a1, a2] 

    html_generator.generate('generate_index.html')


import setuptools
import roborecipe

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="roborecipe",
    version=roborecipe.__version__,
    install_requires=[
        "PyOpenGL",
        "jinja2",
        "pillow",
        "networkx",
    ],
    entry_points={
        'console_scripts': [
            'roborecipe=roborecipe.roborecipe_main:main',
        ],
    },
    author="Erio.Akanuma",
    author_email="e.a@example.com",
    description="generate robot build instruction from scripts",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/eriac/roborecipe",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)


from OpenGL.GL   import *
from OpenGL.GLU  import *
from OpenGL.GLUT import *
from sys import argv
from roborecipe.StlLoader import *

class PartSource:
    def __init__(self):
        self.filename = ""
        self.stl_model = None
        self.position = [0,0,0]
        self.rotation = [0,0,0,0]

class ViewSource:
    def __init__(self):
        self.filename = ""
        self.part_list = []

class ImageGenerator:
    def __init__(self):
        None


def init(): 
   glClearColor(0.0, 0.0, 0.0, 0.0)
#    glShadeModel(GL_FLAT)

def display():
    glClear(GL_COLOR_BUFFER_BIT)
    glColor3f(1.0, 1.0, 1.0)
    glLoadIdentity()                 # clear the matrix 
    # viewing transformation 
    gluLookAt(0.0, 0.0, 600.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0)
    glScalef(1.0, 1.0, 1.0)          # modeling transformation 
    glutWireCube(10.0)

    stl1 = stl_load()
    stl1.load_stl("/home/ubuntu/roborecipe/old_resources/Laser1.stl")
    # stl1.draw()

    glBegin(GL_TRIANGLES)
    for tri in stl1.get_triangles():
        length=tri.normal.length()
        glNormal3f(-tri.normal.x/length,-tri.normal.y/length,-tri.normal.z/length)
        print("try",tri.points[0].x,tri.points[0].y,tri.points[0].z)
        glVertex([tri.points[0].x,tri.points[0].y,tri.points[0].z])
        glVertex([tri.points[1].x,tri.points[1].y,tri.points[1].z])
        glVertex([tri.points[2].x,tri.points[2].y,tri.points[2].z])
    glEnd()


    glFlush()

def reshape(w, h):
   glViewport(0, 0, w, h)
   glMatrixMode(GL_PROJECTION)
   glLoadIdentity()
#    glFrustum(-1.0, 1.0, -1.0, 1.0, 1.5, 2000.0)
   gluPerspective(80, 1, 1.5, 2000)
   glMatrixMode(GL_MODELVIEW)

## ----------------------------------------
if __name__ == '__main__':
    glutInit(sys.argv)
    glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB)
    glutInitWindowSize(640, 640)
    glutInitWindowPosition(100, 100)
    glutCreateWindow("test")             # modify
    init()
    glutDisplayFunc(display)
    glutReshapeFunc(reshape)
    # glutKeyboardFunc(keyboard)         # append
    glutMainLoop()


#!/usr/bin/python3

import collections
import networkx as nx
import copy

from roborecipe.data_type import *

class TreeAnalyzer:
    def __init__(self, component_list, top_assembly_id):
        self.component_list = component_list
        self.top_assembly_id = top_assembly_id

    def getQuantityList(self):
        dl = self._getDuplicateList(self.top_assembly_id)
        return collections.Counter(dl)

    def getDependOrderList(self):
        el = self._getEdgeList(self.top_assembly_id)
        return self._getDependOrder(el)

    def _getDuplicateList(self, component_id):
        part_list = []
        comp = self._getItemFromComponentList(component_id)
        if type(comp) is DataPart:
            part_list.append(comp)
        elif type(comp) is DataAssembly:
            part_list.append(comp)
            for s in comp.step_list:
                for c in s.child_list:
                    l = self._getDuplicateList(c.id)
                    part_list.extend(l)
        return part_list

    def _getItemFromComponentList(self, id):
        for c in self.component_list:
            if c.id == id:
                return c
        return None

    def _getEdgeList(self, component_id, parent=None):
        edge_list = []
        comp = self._getItemFromComponentList(component_id)
        if parent is not None:
            edge_list.append([parent, comp])
        if type(comp) is DataAssembly:
            for s in comp.step_list:
                for c in s.child_list:
                    el = self._getEdgeList(c.id, comp)
                    edge_list.extend(el)
        return edge_list

    def _getDependOrder(self, edge_list):
        graph = nx.DiGraph()
        for e in edge_list:
            graph.add_edge(e[1], e[0])

        top_comp = self._getItemFromComponentList(self.top_assembly_id)
        T = nx.topological_sort(graph)
        return list(T)

class ItemWithTransformList:
    def __init__(self, component, transform_list=[]):
        self.component = component
        self.transform_list = transform_list
        self.step = 0
        self.move = [0, 0, 0]
    def AddParentTransform(self, transform):
        self.transform_list.insert(0, transform)
    def GetWholeTransform(self):
        output = Transform()
        for t in self.transform_list:
            output = output * t
        return output

class RenderTree:
    def __init__(self, component_list, depend_list):
        self.component_list = component_list
        self.depend_list = depend_list
        self.iwtl_list_dict = {}
        for comp in depend_list:
            self.iwtl_list_dict[comp] = self._ExpandComponent(comp)

    def GetItemListWithTransform(self, target_id):
        for comp in self.iwtl_list_dict:
            if comp.id == target_id:
                return self.iwtl_list_dict[comp]

    def _ExpandComponent(self, component):
        output_list = []
        if type(component) is DataPart:
            output_list.append(ItemWithTransformList(component, [Transform()]))
        elif type(component) == DataAssembly:
            step_seq_no = 0
            for s in component.step_list:
                for c in s.child_list:
                    child_item_list = self.GetItemListWithTransform(c.id)
                    move = [c.move.x, c.move.y, c.move.z]
                    for ci in child_item_list:
                        new_ci = copy.deepcopy(ci)
                        new_ci.AddParentTransform(c.transform)
                        new_ci.step = step_seq_no
                        new_ci.move = move
                        output_list.append(new_ci)
                step_seq_no += 1
        return output_list




#!/usr/bin/python3
from roborecipe.TopPageGenerator import * 
from roborecipe.AssemblyAnalyzer import * 


if __name__ == '__main__':
    component_path_list = [
        ComponentPathItem('screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3/roborecipe/pan_10.yaml'),
        ComponentPathItem('screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3', '/home/ubuntu/workspace/roborecipe/sample/screw_m3/roborecipe/hollow_spacer_20.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/bar_plate.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/base_plate.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/side_asm.yaml'),
        ComponentPathItem('sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project', '/home/ubuntu/workspace/roborecipe/sample/sample_project/roborecipe/main_asm.yaml'),
    ]

    cll = ComponentListLoader()
    for path_item in component_path_list:
        cll.load(path_item)

    top_assembly_id = ComponentIdentifier('sample_project', 'main_asm')
    acl = AssemblyComponentListAnalyzer(cll, top_assembly_id)

    td = TopPageData()
    td.title = top_assembly_id.getName()

    for item in acl.getQuantityList(ComponentCategoryEnum.SCREW):
        name_body = item[0].id.getName()
        name_link = item[0].id.getName() + '.html'
        td.mechanical_purchased.add_line(ComponentTableLine(name_body,name_link,item[1],10,item[0].description))

    for item in acl.getQuantityList(ComponentCategoryEnum.LASER_CUT):
        name_body = item[0].id.getName()
        name_link = item[0].id.getName() + '.html'
        td.laser_cut.add_line(ComponentTableLine(name_body,name_link,item[1],20,item[0].description))

    for item in acl.getAssemblyDependencyList():
        name_body = item[0].id.getName()
        name_link = item[0].id.getName() + '.html'
        td.assembly.add_line(ComponentTableLine(name_body,name_link,item[1],0,item[0].description))


    tg = TopPageGenerator(td)
    tg.write('generate_index.html')



#!/usr/bin/python3
from jinja2 import Template
import pathlib
from roborecipe.data_type import * 
import collections

class MechanicalPartsPageData:
    def __init__(self):
        self.title = ""
        self.overwiew_image = ""
        self.description = ""
        self.product_name = ""
        self.product_url = ""
        self.distributor_list = []

class MechanicalPartsPageGenerator:
    def __init__(self, comp):
        self.comp = comp
        template_filepth = pathlib.Path(__file__).parent / 'templates' / 'mechanical_parts.html'
        file = open(template_filepth)
        self.template = Template(file.read()) 

    def write(self, output_base_dir):
        data = MechanicalPartsPageData()
        data.title = self.comp.id.getName() 
        data.overwiew_image = 'images/' + self.comp.id.type_name + '_rigid_body.png'
        data.image_list = self.comp.common.image_list
        data.description = self.comp.common.description
        data.product_name = self.comp.product.name
        data.product_url = self.comp.product.url
        data.distributor_list = self.comp.distributor_list
        rendered_str = self.template.render(data = data)

        output_path = str(output_base_dir) + '/' + self.comp.id.pkg_name + '/' + self.comp.id.type_name + '.html'
        print('write html ' + output_path)
        with open(output_path, mode='w') as f:
            f.write(rendered_str)


# LaserCut 3Dprint
class ProcessPartsPartsPageData:
    def __init__(self):
        self.title = ""
        self.overwiew_image = ""
        self.image_list = []
        self.description = ""
        self.process_data_file = ""
        self.process_material = ""
        self.process_cost = 0

class ProcessPartsPageGenerator:
    def __init__(self, comp):
        self.comp = comp
        template_filepth = pathlib.Path(__file__).parent / 'templates' / 'process_parts.html'
        file = open(template_filepth)
        self.template = Template(file.read())

    def write(self, output_base_dir):
        data = ProcessPartsPartsPageData()
        data.title = self.comp.id.getName() 
        data.overwiew_image = 'images/' + self.comp.id.type_name + '_rigid_body.png'
        data.image_list = self.comp.common.image_list
        data.description = self.comp.common.description
        data.process_data = self.comp.process.data_filepath
        data.process_material = self.comp.process.material
        data.process_cost = self.comp.process.cost
        rendered_str = self.template.render(data = data)

        output_path = str(output_base_dir) + '/' + self.comp.id.pkg_name + '/' + self.comp.id.type_name + '.html'
        print('write html ' + output_path)
        with open(output_path, mode='w') as f:
            f.write(rendered_str)

# Harness
class HarnessPageData:
    def __init__(self):
        self.title = ""
        self.description = ""
        self.table_title = HarnessTableLineData()
        self.table_content_list = []

class HarnessTableLineData:
    def __init__(self, line = '', end1 = '', end2 = ''):
        self.line = line
        self.end1 = end1
        self.end2 = end2

class HarnessPageGenerator:
    def __init__(self, comp):
        self.comp = comp
        template_filepth = pathlib.Path(__file__).parent / 'templates' / 'harness.html'
        file = open(template_filepth)
        self.template = Template(file.read())

    def write(self, output_base_dir):
        data = HarnessPageData()
        data.title = self.comp.id.getName() 
        data.description = self.comp.common.description
        rendered_str = self.template.render(data = data)

        output_path = str(output_base_dir) + '/' + self.comp.id.pkg_name + '/' + self.comp.id.type_name + '.html'
        print('write html ' + output_path)
        with open(output_path, mode='w') as f:
            f.write(rendered_str)

# assembly
class AssemblyPageData:
    def __init__(self):
        self.title = ""
        self.description = ""
        self.step_list = []

class MechanicalPartsStepItem:
    def __init__(self):
        self.seq_no = 0
        self.component_list = []
        self.image_path_list = []

class MechanicalPartsComponentItem:
    def __init__(self):
        self.name = ""
        self.quantity = 0

class AssemblyPageGenerator:
    def __init__(self, comp):
        self.comp = comp
        template_filepth = pathlib.Path(__file__).parent / 'templates' / 'assembly.html'
        file = open(template_filepth)
        self.template = Template(file.read())

    def write(self, output_base_dir):
        data = AssemblyPageData()
        data.title = self.comp.id.getName()
        data.image_list = self.comp.common.image_list
        data.description = self.comp.common.description
        for step_no, step in enumerate(self.comp.procedure_list):
            step_data = MechanicalPartsStepItem()
            step_data.seq_no = step_no
            comp_id_list = []
            for c in step.component_list:
                comp_id_list.append(c.id)
            quantity_list = collections.Counter(comp_id_list)
            for comp_id in quantity_list:
                comp_data = MechanicalPartsComponentItem()
                comp_data.name = comp_id.getName()
                comp_data.quantity = quantity_list[comp_id]
                step_data.component_list.append(comp_data)
            view_size = len(step.view_list)
            for i in range(view_size):
                step_data.image_path_list.append('images/' + self.comp.id.type_name + '_' + str(step_no) + '_' + str(i) + '.gif')
            data.step_list.append(step_data)
        rendered_str = self.template.render(data = data)

        output_path = str(output_base_dir) + '/' + self.comp.id.pkg_name + '/' + self.comp.id.type_name + '.html'
        print('write html ' + output_path)
        with open(output_path, mode='w') as f:
            f.write(rendered_str)

if __name__ == '__main__':
    mpd = MechanicalPartsPageData()
    mpg = MechanicalPartsPageGenerator(mpd)
    mpg.write('generate_index.html')